<?php
/**
 * Plugin Name: google-dynamic-password
 * Plugin URI: http://www.myluoluo.com
 * Description: 登录时验证动态密码，以用来提升WordPress安全性。
 * Version: 2.0
 * Author: LuoLuo
 * Author URI: http://www.myluoluo.com
 */

/* 注册激活插件时要调用的函数 */ 
register_activation_hook( __FILE__, 'dynamic_password_install');   

/* 注册停用插件时要调用的函数 */ 
register_deactivation_hook( __FILE__, 'dynamic_password_remove' );  


function dynamic_password_install() {
    require_once 'GoogleAuthenticator.php';
    $ga = new PHPGangsta_GoogleAuthenticator();
    $secret = $ga->createSecret();
    /* 在数据库的 wp_options 表中添加一条记录，第二个参数为默认值 */ 
    add_option("dynamic_password_secretkey", $secret, '', 'yes');  
}

function dynamic_password_remove() {  
    /* 删除 wp_options 表中的对应记录 */ 
    delete_option('dynamic_password_secretkey');  
}

if( is_admin() ) {
    /*  利用 admin_menu 钩子，添加菜单 */
    add_action('admin_menu', 'dynamic_password_menu');
}

function dynamic_password_menu() {
    /* add_options_page( $page_title, $menu_title, $capability, $menu_slug, $function);  */
    /* 页名称，菜单名称，访问级别，菜单别名，点击该菜单时的回调函数（用以显示设置页面） */
    add_options_page('Google动态密码验证设置', 'Google动态密码验证', 'administrator','google_authconfig', 'dynamic_password_html_page');
}

function dynamic_password_html_page() {
    require_once 'GoogleAuthenticator.php';
    $ga = new PHPGangsta_GoogleAuthenticator();
    $secret = get_option('dynamic_password_secretkey');
    $qrCodeUrl = $ga->getQRCodeGoogleUrl(get_option('blogname'), $secret); //第一个参数是"标识",第二个参数为"安全密匙SecretKey" 生成二维码信息
    $oneCode = $ga->getCode($secret); //服务端计算"一次性验证码"
?>
    <div>
        <h2>Google动态密码验证设置</h2>
        <form method="post" action="options.php">
            <?php /* 下面这行代码用来保存表单中内容到数据库 */ ?>
            <?php wp_nonce_field('update-options'); ?>
            
            <table class="form-table">
                <tbody>
                    <tr>
                        <th scope="row"><label for="dynamic_password_secretkey">安全密匙SecretKey：</label></th>
                        <td><input name="dynamic_password_secretkey" id="dynamic_password_secretkey" type="text" value="<?php echo $secret; ?>" class="regular-text" /><p>动态密码生成时必要的参数，插件安装时会自动生成，请妥善保管该项。</p></td>
                    </tr>
                    <tr>
                        <th scope="row"><label>QR-Code：</label></th>
                        <td>
                            <img src="<?php echo $qrCodeUrl; ?>"></img>
                            <p>使用Google验证器即可添加，二维码生成为Google提供。</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><label>当前服务器时区：</label></th>
                        <td><input readonly="readonly" type="text" value="<?php echo date_default_timezone_get(); ?>" class="regular-text" /><p>若服务器时间与验证终端时间相差过大将导致验证失败。</p></td>
                    </tr>
                    
                    <tr>
                        <th scope="row"><label>当前服务器时间(PRC)：</label></th>
                        <td><input readonly="readonly" type="text" value="<?php date_default_timezone_set("PRC");echo date('Y-m-d H:i:s',time()); ?>" class="regular-text" /><p>若服务器时间与验证终端时间相差过大将导致验证失败。</p></td>
                    </tr>
                    <tr>
                        <th scope="row"><label>最新密码：</label></th>
                        <td><input readonly="readonly" type="text" value="<?php echo $oneCode; ?>" class="regular-text" /></td>
                    </tr>
                </tbody>
            </table>
            
            <p>
                <input type="hidden" name="action" value="update" />
                <input type="hidden" name="page_options" value="dynamic_password_secretkey" />
                <input type="submit" value="保存" class="button-primary" />
            </p>
        </form>
    </div>
    
<?php  
}

//用户名与密码非空时绑定验证
if(!empty($_POST["log"]) & !empty($_POST["pwd"])){
    add_filter('authenticate', 'CheckPassword', 30, 3);
}
add_action("login_form","FormPassword",0);

//动态密码验证
function CheckPassword($userobj,$user_name)
{
    global $_POST;
    //空提交验证
    $ip  = $_SERVER["REMOTE_ADDR"];
    if (empty($_POST["dyn-pass"])){
        $errobj=new WP_Error('broke', __("<strong>错误</strong>：请输入动态密码！！"));
        return $errobj;
    }
    
    require_once 'GoogleAuthenticator.php';
    $ga = new PHPGangsta_GoogleAuthenticator();
    // 最后一个参数 为容差时间,这里是2 那么就是 2* 30 sec 一分钟.
    $secret = get_option('dynamic_password_secretkey');
    $checkResult = $ga->verifyCode($secret, trim($_POST["dyn-pass"]), 2);

    if ($checkResult) {
        return $userobj;
    } else {
        $errobj=new WP_Error('broke', __("<strong>错误</strong>：动态密码验证失败！"));
        return $errobj;
    }
}

//登录表单
function FormPassword()
{
    echo '<p><label for="dyn-pass">动态密码<br/><input type="text" name="dyn-pass" id="dyn-pass" class="input" value="" size="20"/></label></p>';
}
?>
